using System;
using System.Collections; 
using System.Runtime.InteropServices;
using System.Text;
using Microsoft.Win32;
using System.Globalization;
using System.Reflection;
using Win32;
using System.Windows.Forms;


namespace ContextExtend
{
	/// <summary>
	/// Summary description for ShellExtension.
	/// </summary>
	[Guid("C6DAC063-E136-4a3d-B237-E40759123084"), ComVisible(true)]
	public class ShellExtension: IContextMenu, IShellExtInit

	{
		public ShellExtension()
		{
		}

		private ShellMenuContextItem[]	m_shellMenuContextItems = { new CopyPathToClipboard(),
																	new UnlockFiles()
																  };
		#region Registration
		private static string clsid = "{C6DAC063-E136-4a3d-B237-E40759123084}";

		[System.Runtime.InteropServices.ComRegisterFunctionAttribute()]
		public static void RegisterServer(Type t) 
		{
			try 
			{
				RegistryKey root;
				RegistryKey key;

				root = Registry.ClassesRoot;
				key = root.CreateSubKey(@"*\shellex\ContextMenuHandlers\ContextExtend");
				key.SetValue("", clsid);
				key.Close();

				key = root.CreateSubKey(@"Folder\shellex\ContextMenuHandlers\ContextExtend");
				key.SetValue("", clsid);
				key.Close();

				// For Win NT
				root = Registry.LocalMachine;
				key = root.OpenSubKey(@"Software\Microsoft\Windows\CurrentVersion\Shell Extensions\Approved", true);
				key.SetValue(clsid, "ContextExtend");
				key.Close();
			}
			catch (Exception e)
			{
				MessageBox.Show(e.Message);
			}
		}
		#endregion

		#region IContextMenu Members

		public int QueryContextMenu(uint hmenu, uint iMenu, int idCmdFirst, int idCmdLast, uint uFlags)
		{
			int iMin = m_shellMenuContextItems.GetLength(0);
			int iMax = - 1;
			try 
			{
				uint		  uiSelectedFiles = DllImports.DragQueryFile(hDrop, 0xFFFFFFFF, null, 0);

				if ( uiSelectedFiles == 0xFFFFFFFF )
					return 0;

				DllImports.InsertMenu( hmenu, iMenu++, (uint)(MF.BYPOSITION|MF.SEPARATOR), 0, null );

				// Clear all lists
				for ( int i = 0; i < m_shellMenuContextItems.GetLength(0); i++ )
				{
					m_shellMenuContextItems[i].fileList.Clear();
					m_shellMenuContextItems[i].directoryList.Clear();
				}

				for ( uint i = 0; (i < uiSelectedFiles); i ++ )
				{
					StringBuilder		sb = new StringBuilder(1024);

					if ( DllImports.DragQueryFile(hDrop, i, sb, sb.Capacity + 1) > 0  ) 
					{
						// If it's a directory
						if ( System.IO.Directory.Exists( sb.ToString() ) )
						{
							System.IO.DirectoryInfo	directoryInfo = new System.IO.DirectoryInfo(sb.ToString());

							for ( int j = 0; j < m_shellMenuContextItems.GetLength(0); j++ )
								if ( m_shellMenuContextItems[j].Handles( directoryInfo ) )
									m_shellMenuContextItems[j].directoryList.Add(directoryInfo);
						}
						else if (System.IO.File.Exists( sb.ToString() ) )
						{
							System.IO.FileInfo	fileInfo = new System.IO.FileInfo( sb.ToString() );

							for ( int j = 0; j < m_shellMenuContextItems.GetLength(0); j++ )
								if ( m_shellMenuContextItems[j].Handles( fileInfo ) )
									m_shellMenuContextItems[j].fileList.Add(fileInfo);
						}
					}
				}

				// Add all possible item 
				for ( int i = 0; i < m_shellMenuContextItems.GetLength(0); i++ )
				{
					if ( (m_shellMenuContextItems[i].fileList.Count > 0) || ( m_shellMenuContextItems[i].directoryList.Count > 0 ) )
					{
						if (i < iMin)
							iMin = i;

						if ( i > iMax )
							iMax = i;
						DllImports.InsertMenu( hmenu, iMenu++, (uint)(MF.BYPOSITION|MF.STRING), (uint)(idCmdFirst + i), m_shellMenuContextItems[i].name );
					}
				}

				DllImports.InsertMenu( hmenu, iMenu++, (uint)(MF.BYPOSITION|MF.SEPARATOR), 0 , null );
			}
			catch( System.Exception e )
			{
				MessageBox.Show("Error in QueryContextMenu:" + e.Message, e.StackTrace );
			}
			return (iMax - iMin) + 1;
		}

		public void InvokeCommand(ref CMINVOKECOMMANDINFO pici)
		{
			try
			{
				m_shellMenuContextItems[(int) (pici.lpVerb & 0xFFFF)].Run(ref pici );
			}
			catch
			{
			}
		}

		public int GetCommandString (UIntPtr idCmd, uint uType, UIntPtr pwReserved, StringBuilder pszName, ushort cchMax)
		{
			pszName = new StringBuilder("Et ta soeur".Substring(1, cchMax - 1));
			switch(uType)
			{
				case (ushort)GCS.VERB:
				{
					pszName = new StringBuilder("ContextExtend".Substring(1, cchMax - 1));
					break;
				}
				case (ushort)GCS.HELPTEXT:
				{
					pszName = new StringBuilder("ContextExtend".Substring(1, cchMax - 1));
					break;
				}
			}
			return 0;
		}

		#endregion

		#region IShellExtInit Members

		Win32.IDataObject dataObject = null;
		uint hDrop = 0;

		public int Initialize(IntPtr pidlFolder, IntPtr lpdobj, uint hKeyProgID)
		{
			try
			{
				dataObject = null;
				if (lpdobj != (IntPtr)0)
				{
					dataObject = 
						(Win32.IDataObject)Marshal.GetObjectForIUnknown(lpdobj);
					FORMATETC fmt  = new FORMATETC();
					fmt.cfFormat   = CLIPFORMAT.CF_HDROP;
					fmt.ptd     = 0;
					fmt.dwAspect   = DVASPECT.DVASPECT_CONTENT;
					fmt.lindex    = -1;
					fmt.tymed    = TYMED.TYMED_HGLOBAL;
					STGMEDIUM medium = new STGMEDIUM();
					dataObject.GetData(ref fmt, ref medium);
					hDrop     = medium.hGlobal;
				}
			}
			catch(Exception)
			{
			}
			return 0;
		}

		#endregion

		#region ErrorMessage
		private void DisplayErrorMessage( string strMethodName, System.Exception e )
		{
			MessageBox.Show("Error in " + strMethodName + ":" + e.Message, e.StackTrace );
		}
		#endregion
	}
}
